@@ -38,6 +38,26 @@ def RegisterAllImplementations(register_callback):
   register_callback('nssdb', 'shadow', NssDbShadowHandler)
 
 
+# TODO: Move this function and the use below it can be used for all caches.
+def is_valid_unix_name(name):
+  """
+  Return False if name has characters that are not OK for Unix usernames,
+  True otherwise.
+
+  Unix has certain naming restrictions for user names in passwd, shadow, etc.
+  Here we take a conservative approach and only blacklist a few characters.
+
+  Args:
+   name: name to test
+
+  Returns: True if the name is OK, False if it contains bad characters.
+  """
+  if any(c in name for c in [" ", ":", "\n"]):
+    return False
+  else:
+    return True
+
+
 class NssDbCache(caches.Cache):
   """An implementation of a Cache specific to nss_db.
 
@@ -290,6 +310,8 @@ class NssDbPasswdHandler(NssDbCache):
     Returns:
       Nothing
     """
+    if not is_valid_unix_name(entry.name):
+      return
     password_entry = '%s:%s:%d:%d:%s:%s:%s' % (entry.name, entry.passwd,
                                                entry.uid, entry.gid,
                                                entry.gecos, entry.dir,
@@ -351,6 +373,8 @@ class NssDbPasswdHandler(NssDbCache):
     Returns:
       a list of strings
     """
+    if not is_valid_unix_name(entry.name):
+      return []
     return ['.%s' % entry.name,
             '=%d' % entry.uid]
 
@@ -380,6 +404,8 @@ class NssDbGroupHandler(NssDbCache):
     Returns:
       Nothing
     """
+    if not is_valid_unix_name(entry.name):
+      return
     grent = '%s:%s:%d:%s' % (entry.name, entry.passwd, entry.gid,
                              ','.join(entry.members))
     # Write to makedb with each key
@@ -433,6 +459,8 @@ class NssDbGroupHandler(NssDbCache):
     Returns:
       a list of strings
     """
+    if not is_valid_unix_name(entry.name):
+      return []
     return ['.%s' % entry.name,
             '=%d' % entry.gid]
 
@@ -463,6 +491,8 @@ class NssDbShadowHandler(NssDbCache):
     Returns:
       Nothing
     """
+    if not is_valid_unix_name(entry.name):
+      return
     # If the field is None, then set to empty string
     shadow_entry = '%s:%s:%s:%s:%s:%s:%s:%s:%s' % (entry.name,
                                                    entry.passwd,
@@ -534,4 +564,6 @@ class NssDbShadowHandler(NssDbCache):
     Returns:
       a list of strings
     """
+    if not is_valid_unix_name(entry.name):
+      return []
     return ['.%s' % entry.name]
