@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2007 Google Inc.
 #
 # This program is free software; you can redistribute it and/or
@@ -173,8 +171,12 @@ class PidFile(object):
     commandline including the string 'nsscache'.
     """
     # Grab the pid
-    pid = self._file.read().strip()
-    pid = int(pid)
+    pid_content = self._file.read()
+    try:
+      pid = int(pid_content.strip())
+    except (AttributeError, ValueError), e:
+      self.log.warning('Not sending TERM, could not parse pid file content: %r', pid_content)
+      return
 
     self.log.debug('retrieved pid %d' % pid)
 
