@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2007 Google Inc.
 #
 # This program is free software; you can redistribute it and/or
@@ -169,7 +167,7 @@ class TestPasswdMap(unittest.TestCase):
     """Test setting/getting of timestamps on maps."""
     m = passwd.PasswdMap()
     # we only work in whole-second resolution
-    now = time.gmtime(time.time())
+    now = int(time.time())
     
     m.SetModifyTimestamp(now)
     self.assertEqual(now, m._last_modification_timestamp)
