@@ -690,6 +690,7 @@ class GroupUpdateGetter(UpdateGetter):
     if 'groupregex' in conf:
       self.groupregex = re.compile(self.conf['groupregex'])
     self.essential_fields = ['cn']
+    self.log = logging.getLogger(self.__class__.__name__)
 
   def CreateMap(self):
     """Return a GroupMap instance."""
@@ -703,7 +704,9 @@ class GroupUpdateGetter(UpdateGetter):
     gr.name = obj['cn'][0]
     # group passwords are deferred to gshadow
     gr.passwd = '*'
+    base = self.conf.get("base")
     members = []
+    group_members = []
     if 'memberUid' in obj:
       if hasattr(self, 'groupregex'):
         members.extend(''.join([x for x in self.groupregex.findall(obj['memberUid'])]))
@@ -712,6 +715,9 @@ class GroupUpdateGetter(UpdateGetter):
     elif 'member' in obj:
       for member_dn in obj['member']:
         member_uid = member_dn.split(',')[0].split('=')[1]
+        # Note that there is not currently a way to consistently distinguish
+        # a group from a person
+        group_members.append(member_uid)
         if hasattr(self, 'groupregex'):
           members.append(''.join([x for x in self.groupregex.findall(member_uid)]))
         else:
@@ -723,6 +729,7 @@ class GroupUpdateGetter(UpdateGetter):
 
     gr.gid = int(obj['gidNumber'][0])
     gr.members = members
+    gr.groupmembers = group_members
 
     return gr
 
@@ -741,6 +748,26 @@ class GroupUpdateGetter(UpdateGetter):
               uidmembers.extend(obj['uid'])
         del gr.members[:]
         gr.members.extend(uidmembers)
+    
+    _group_map = {i.name: i for i in data_map}
+    
+    def _expand_members(obj, visited=None):
+      """Expand all subgroups recursively"""
+      for member_name in obj.groupmembers:
+        if member_name in _group_map and member_name not in visited:
+          gmember = _group_map[member_name]
+          for member in gmember.members:
+            if member not in obj.members:
+              obj.members.append(member)
+          for submember_name in gmember.groupmembers:
+            if submember_name in _group_map and submember_name not in visited:
+              visited.append(submember_name)
+              _expand_members(_group_map[submember_name], visited)
+    
+    if self.conf.get("nested_groups"):
+      self.log.info("Expanding nested groups")
+      for gr in data_map:
+        _expand_members(gr, [gr.name])
 
 
 class ShadowUpdateGetter(UpdateGetter):
